
import os,sys,operator
import hashlib

sitebase = "https://xcquals.com/"
dirbase = "./"

def regional_place_formvarname(rname, place):
  return rname + "-" + str(place)

def regional_noplacename(rname):
  return rname + "-0"

def get_baseurl(year, gender):
  return sitebase + year + "/" + gender + "/"

def get_basedir(year, gender):
  return os.path.join(dirbase, year, gender)

# hash results to get a short id string
def get_idstr(year, gender, regs):
  slist = [year,gender]
  for rname,rteams in sorted(regs.items()):
    slist.append(rname)
    slist.extend(rteams)
  s = ''.join(slist)
  return hashlib.md5(s.encode("utf-8")).hexdigest()


def get_dir(year, gender, idstr):
  return os.path.join(dirbase, year, gender, "quals", idstr)

def get_url_base(year, gender, idstr):
  return sitebase + year + "/" + gender + "/quals/" + idstr + "/"

class Meet:
  def __init__(self, name, teams, date="3000-01-01"):
    self.name = name
    self.date = date
    self.teams = teams  # in order of place
    self.team_scores = [0]*len(teams)  # ditto, currently unused
    self.loc = ""    # location, currently unused
    self.rosters = {}  # map team name to list of runner names, currently unused
    self.inds = []  # list of runner names in order of place, currently unused
    self.ind_scores = []  # list of runner scores (after taking out 8th+ runners), currently unused

def get_meet_info(filename):
  try:
    with open(filename) as f:
      lines = [line.strip() for line in f.readlines()]
    name = lines[0]
    date = lines[1]
    ind = 1
    # find the next empty line and skip it (possibly the second)
    while ind < len(lines) and len(lines[ind]) > 0:
      ind += 1
    # format of each line: 1 Team Name
    teams = [line[line.index(" ")+1:].strip() for line in lines[ind+1:] if len(line.strip()) > 0]
    return name, date, teams
  except:
    raise Exception("Could not parse file " + filename)


# Read in season results
# return dict mapping meet name to pair (date, list-of-teams-in-order-of-place)
# year is "2018" or so on (a string)
# gender is "m" or "w"
def get_season_results(year, gender):
  meet_res = dict()
  seasons_dir = os.path.join(dirbase, year, gender, "meets")
  try:
    meets_files = os.listdir(seasons_dir)
  except:
    raise Exception("Could not find folder " + seasons_dir)
  for filename in meets_files:
    name, date, teams = get_meet_info(os.path.join(seasons_dir,filename))
    meet_res[name] = (date, teams)
  return meet_res

# Read in regional results
# return dict mapping region name to list of teams in order of place
# year is 2018 or so on
# gender is "m" or "w"
def get_regional_results(year, gender):
  regional_res = dict()
  regions_dir = os.path.join(dirbase, year, gender, "regionals")
  try:
    regions_files = os.listdir(regions_dir)
  except:
    raise Exception("Could not find folder " + regions_dir)
  for filename in regions_files:
    name, date, teams = get_meet_info(os.path.join(regions_dir,filename))
    regional_res[name] = teams
  return regional_res


## print out list of all teams (so you can inspect for typos)
def print_teams(meets, regionals):
  teamlist = {}
  for _,meet in regionals.items():
    for team in meet:
      teamlist[team] = True
  for _,meetpr in meets.items():
    for team in meetpr[1]:
      teamlist[team] = True
  for t in sorted(teamlist.keys()):
    print(t)

## print out the meets and regionals
def print_results(meets, regionals):
  print("regional_results = {")
  for rname,rteams in sorted(regionals.items()):
    print('  "' + rname + '": ["' + '", "'.join(rteams) + '"],')
  print("}\n")

  meets_by_date = [(pr[0], meetname, pr[1]) for meetname, pr in meets.items()]
  meets_by_date.sort()
  print("season_results = {")
  for date,meetname,teamlist in meets_by_date:
    print('  "' + meetname + '": ("' + date + '",\n      ["' + '", "'.join(teamlist) + '"]),')
  print("}\n")
  

# given the meet results dictionary,
# return a Meet Object dictionary
def get_meets(meet_res):
  meets = dict()
  for mname, pr in meet_res.items():
    m = Meet(mname, pr[1], pr[0])
    meets[mname] = m
  return meets

def get_regionals(regional_res):
  regionals = dict()
  for rname, teamlist in regional_res.items():
    m = Meet(rname + " regional", teamlist)
    regionals[rname] = m
  return regionals


def get_regions_list(year, gender):
  filename = os.path.join(dirbase, year,gender,"regions.txt")
  try:
    with open(filename) as f:
      lines = [l.strip() for l in f.readlines()]
    return lines
  except:
    raise Exception("Could not find file " + filename)


usage_str = "When run from the command line, prints all teams and results.\nUsage:\n    python3 util.py yyyy w\n    or\n    python3 util.py yyyy m\n"
if __name__ == "__main__":
  if len(sys.argv) != 3:
    print(usage_str)
    exit(0)
  try:
    year = sys.argv[1]
    gender = sys.argv[2]
    season = get_season_results(year, gender)
    regs = get_regional_results(year, gender)
    print_results(season, regs)
    print("\n-------------------------------------------------------------------\n")
    print_teams(season, regs)
  except:
    print(usage_str)

